GLOBAL_STASH_PREFIX = 'gitsy_global_stash'

DEFAULT_COMMIT = 'fast commit'

BRANCH_CONTEXT = {
    'type': 'branch',
    'status': 'enabled',
    'working-on': []
}

GLOBAL_CONTEXT = {
    'type': 'global',
    'local': False
}
